#!/usr/bin/python

# << insert correctly modified ansible_preamble here

import json
import subprocess

from ansible.module_utils.basic import AnsibleModule

from ansible.module_utils.am2cm_commons.cdp_cluster import CDPCluster


def run_module():
    # define available arguments/parameters a user can pass to the module
    module_args = dict(
        ambari_configuration=dict(type='dict', required=True),
        cm_configuration=dict(type='dict', required=True),
        am2cm_configuration=dict(type='dict', required=True)
    )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
        original_message='',
        message=''
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True
    )
    result['original_message'] = json.dumps(module.params)  # concatenate the params here, if there were any...


    cm_hostname = module.params['cm_configuration']['cloudera_manager_hostname']
    cm_port = module.params['cm_configuration']['cloudera_manager_port']
    cm_username = module.params['cm_configuration']['cloudera_manager_admin_username']
    cm_password = module.params['cm_configuration']['cloudera_manager_admin_password']
    cm_protocol = module.params['cm_configuration']['cloudera_manager_protocol']
    cm_cluster_name = module.params['cm_configuration']['cluster_name']
    is_cm_https = cm_protocol == 'https'

    # check if cm-cluster exists. If so then return error msg
    cluster = CDPCluster(
        hostname=cm_hostname, port=cm_port, is_https=is_cm_https, verify_ssl=False,
        username=cm_username, password=cm_password, cluster_name=cm_cluster_name)

    cluster_exist = cluster.is_cluster_exist()

    # if the user is working with this module in only check mode we do not
    # want to make any changes to the environment, just return the current
    # state with no modifications
    if module.check_mode:
        result['changed'] = not cluster_exist
        module.exit_json(**result)

    if cluster_exist:
        result['changed'] = False
        module.fail_json(msg='CDP cluster exists! Please delete it first!', **result)


    ambari_hostname = module.params['ambari_configuration']['hostname']
    ambari_port = module.params['ambari_configuration']['port']
    ambari_username = module.params['ambari_configuration']['username']
    ambari_password = module.params['ambari_configuration']['password']
    ambari_cluster_name = module.params['ambari_configuration']['cluster_name']
    ambari_protocol = module.params['ambari_configuration']['protocol']

    script_path = module.params['am2cm_configuration']['script_path']
    source_version = module.params['am2cm_configuration']['source_version']
    target_version = module.params['am2cm_configuration']['target_version']
    deployment_template_file = module.params['am2cm_configuration']['deployment_template_file']
    skip_pre_upgrade = module.params['am2cm_configuration']['skip_pre_upgrade']
    user_setting_file = module.params['am2cm_configuration']['user_setting_file']

    args = [f'{script_path}/am2cm.sh',
            '--ambari-server', ambari_hostname,
            '--ambari-port', str(ambari_port),
            '--ambari-username', ambari_username,
            '--ambari-password', ambari_password,
            '--ambari-cluster-name', ambari_cluster_name,
            '--source-version', source_version,
            '--target-version', target_version,
            '--deployment-template-file', deployment_template_file,
            '--silent',
            '--ambari-is-https' if ambari_protocol == 'https' else '',
            '--skip-pre-upgrade' if skip_pre_upgrade else '',
            '--cm-server', cm_hostname,
            '--cm-port', str(cm_port),
            '--cm-username', cm_username,
            '--cm-password', cm_password,
            '--cm-is-https' if is_cm_https else '',
            '--user-setting-file', user_setting_file
            ]

    process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = process.communicate()
    rc = process.returncode

    if rc != 0:
        result['stderr'] = stderr
        module.fail_json(msg='Exception when calling am2cm.sh', **result)

    result['stdout'] = stdout

    # use whatever logic you need to determine whether or not this module
    # made any modifications to your target
    result['changed'] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()
